unit joinu;

// Please, don't delete this comment. \\
(*
  Copyright Owner: coltishWARE (Kevin Niehage)
  Copyright Year : 2008

  Unit   : joinu (platform independant)
  Version: 1.12

  Contact E-Mail            : development@coltishWARE.com
  Contact Mobile            : +49-(0)-177-69-4321-8
  Contact Telephone Berlin  : +49-(0)-30-69-20-96-78
  Contact Telephone Cottbus : +49-(0)-355-28925-88-17-55
  Contact Telephone Mannheim: +49-(0)-621-483-4664
*)
// Please, don't delete this comment. \\

(*
  Description:

  This unit together with splitu implements a functionality to join/split
  a file to/from another file.
*)

(*
  Change Log:

  [Version 1.12] (17.09.2008: inital release with new header)
  - new coltishWARE header is in place now
*)

interface

uses
  SysUtils;

const
  CDefaultBufferSize = 51200;

function JoinFile(const AMainFileName : String; const AJoinedFileName : String; const ABufferSize : LongWord = CDefaultBufferSize) : Boolean;

implementation

function JoinFile(const AMainFileName : String; const AJoinedFileName : String; const ABufferSize : LongWord = CDefaultBufferSize) : Boolean;
var
  Buffer           : PChar;
  BufferSize       : LongInt;
  DoneCount        : LongInt;
  Index            : LongInt;
  JoinedFileHandle : LongInt;
  JoinedFileSize   : LongInt;
  HelpLongInt      : LongInt;
  HelpString       : String;
  MainFileHandle   : LongInt;
  MainFileSize     : LongInt;
  WriteCount       : LongInt;
begin
  Result := false;

  if (FileExists(AMainFileName) and FileExists(AJoinedFileName)) then
  begin
    MainFileHandle := FileOpen(AMainFileName, fmOpenReadWrite or fmShareDenyNone);
    if (MainFileHandle >= 0) then
    begin
      try
        MainFileSize := FileSeek(MainFileHandle, 0, 2);
        FileSeek(MainFileHandle, 0, 0);

        JoinedFileHandle := FileOpen(AJoinedFileName, fmOpenRead or fmShareDenyNone);
        if (JoinedFileHandle >= 0) then
        begin
          try
            JoinedFileSize := FileSeek(JoinedFileHandle, 0, 2);
            FileSeek(JoinedFileHandle, 0, 0);

            if (JoinedFileSize > 0) then
            begin
              FileSeek(MainFileHandle, 0, 2);
              FileSeek(JoinedFileHandle, 0, 0);

              if (JoinedFileSize < ABufferSize) then
                BufferSize := JoinedFileSize
              else
              begin
                if (ABufferSize < 1) then
                  BufferSize := CDefaultBufferSize
                else
                  BufferSize := ABufferSize;
              end;

              GetMem(Buffer, BufferSize);
              try
                WriteCount := 0;
                for Index := 1 to (JoinedFileSize div BufferSize) do
                begin
                  DoneCount := FileRead(JoinedFileHandle, Buffer^, BufferSize);
                  if (DoneCount = BufferSize) then
                  begin
                    DoneCount := FileWrite(MainFileHandle, Buffer^, BufferSize);
                    if (DoneCount = BufferSize) then
                      WriteCount := WriteCount + BufferSize;
                  end;
                end;

                if ((JoinedFileSize mod BufferSize) <> 0) then
                begin
                  DoneCount := FileRead(JoinedFileHandle, Buffer^, (JoinedFileSize mod BufferSize));
                  if (DoneCount = (JoinedFileSize mod BufferSize)) then
                  begin
                    DoneCount := FileWrite(MainFileHandle, Buffer^, (JoinedFileSize mod BufferSize));
                    if (DoneCount = (JoinedFileSize mod BufferSize)) then
                      WriteCount := WriteCount + (JoinedFileSize mod BufferSize);
                  end;
                end;

                Result := (WriteCount = JoinedFileSize);
              finally
                FreeMem(Buffer, BufferSize);
              end;
            end;
          finally
            FileClose(JoinedFileHandle);
          end;

          HelpLongInt := MainFileSize;
          HelpString  := '';
          while (HelpLongInt <> 0) do
          begin
            HelpString  := Chr(HelpLongInt mod Succ(High(Byte))) + HelpString;
            HelpLongInt := (HelpLongInt div Succ(High(Byte)));
          end;
          HelpString := Chr(Low(Byte)) + HelpString + Chr(Succ(Length(HelpString)));

          FileSeek(MainFileHandle, 0, 2);
          DoneCount := FileWrite(MainFileHandle, HelpString[1], Length(HelpString));
          Result := Result and (DoneCount = Length(HelpString));
        end;
      finally
        FileClose(MainFileHandle);
      end;
    end;
  end;
end;

end.
